#ifndef _TUX_MODULE_H
#define _TUX_MODULE_H

/*
 * TUX - Integrated HTTP layer and Object Cache
 *
 * Copyright (C) 2000, Ingo Molnar <mingo@redhat.com>
 *
 * module.h: user-space portions of the HTTP module API
 */

#include "tux.h"

extern char *TUXAPI_docroot;
extern char *TUXAPI_version;

extern int tux (unsigned int action, user_req_t *req);

extern void * TUXAPI_malloc_shared (unsigned int len);

#define BUG()								\
do {									\
	printf("TUX BUG at %d:%s!\n", __LINE__, __FILE__);		\
	tux(TUX_ACTION_STOPTHREAD, NULL);				\
	tux(TUX_ACTION_SHUTDOWN, NULL);					\
	*(int*)0=0;							\
	exit(-1);							\
} while (0)

#define LOCK_PREFIX "lock ; "

#define barrier() __asm__ __volatile__("": : :"memory")
struct __dummy { unsigned long a[100]; };
#define ADDR (*(volatile struct __dummy *) addr)

extern __inline__ int test_and_set_bit(int nr, volatile void * addr)
{
	int oldbit;

	__asm__ __volatile__( LOCK_PREFIX
		"btsl %2,%1\n\tsbbl %0,%0"
		:"=r" (oldbit),"=m" (ADDR)
		:"Ir" (nr));
	return oldbit;
}

extern __inline__ void tux_down (int *sem)
{
	while (test_and_set_bit(0, sem))
		barrier();
}

extern __inline__ void tux_up (int *sem)
{
	*((volatile int *)sem) = 0;
}

#define TUX_DEBUG 0

#if TUX_DEBUG
# define Dprintk(x...) do { printf(x); fflush(stdout); } while (0)
#else
# define Dprintk(x...) do { } while (0)
#endif

#define HTTP_TRACE(req,info) Dprintk("HTTP trace at %s:%d, request %p, info %d, event %d (meth:%d, query:%s, cookies:%s).\n", __FILE__, __LINE__, (req)->id, info, (req)->event, (req)->http_method, (req)->query, (req)->cookies_len ? (req)->cookies : "<no cookies>")

#endif
